package simulator.elevatormodules;

import jSimPack.SimTime;
import simulator.framework.*;

public abstract class Button extends Module implements TimeSensitive {

    private boolean pressed = false;
    private final SimTime defaultPressTime;
    private final Timer timer;

    public Button(String name, SimTime defaultPressTime, SimTime period, boolean verbose) {
        super(period, name, verbose);
        this.defaultPressTime = defaultPressTime;
        timer = new Timer(this);
    }

    //override this method to implement network message interfaces for different buttons
    protected abstract void sendButtonMessage();

    public void press() {
        press(defaultPressTime);
    }

    public void press(SimTime pressTime) {
        if (!pressTime.isPositive()) {
            throw new RuntimeException("Press time must be greater than 0, not " + pressTime);
        }
        pressed = true;
        sendButtonMessage();
        timer.start(pressTime);
    }

    /**
     * Use for manual release
     */
    public void release() {
        pressed = false;
        sendButtonMessage();
    }

    /**
     * When timer expires, release the button.
     * @param callbackData - not used
     */
    public void timerExpired(Object callbackData) {
        pressed = false;
        sendButtonMessage();
    }




    public boolean isPressed() {
        return pressed;
    }

    @Override
    public String toString() {
        return name;
    }


}
