package simulator.elevatormodules;

import jSimPack.SimTime;
import simulator.elevatorcontrol.MessageDictionary;
import simulator.framework.NetworkConnection;
import simulator.framework.ReplicationComputer;
import simulator.payloads.*;

/**
 * Detects the state of the soda chute and send a network message.
 */

public class EmptySensor extends Module
{
    private SodaChutePayload sodaChuteState;
    
    private EmptyCanPayloadTranslator mEmpty;


    /**
     * Creates a sensor that detects when the car is near the specified floor.
     * When the Car position is between <code>bottom</code> and
     * <code>top</code>, inclusive, the Car is considered to be "at" this
     * floor.
     *
     * @param period
     * time in microseconds between AtFloor messages
     */
    public EmptySensor (SimTime period, int index) {
        super(period, "Empty["+index+"]", false);

        sodaChuteState = new SodaChutePayload(index);
        physicalConnection.registerEventTriggered(sodaChuteState);
        physicalConnection.registerTimeTriggered(sodaChuteState);

        CanMailbox p = new CanMailbox(MessageDictionary.EMPTY_BASE_CAN_ID + ReplicationComputer.computeReplicationId(index));
        mEmpty = new EmptyCanPayloadTranslator(p, index);
        canNetworkConnection.sendTimeTriggered(p, period);
    }

    @Override
    public void receive(SodaChutePayload msg) {
        mEmpty.setValue(sodaChuteState.sodaCount == 0);
    }
    



}
