package simulator.elevatormodules;

import jSimPack.SimTime;
import jSimPack.SimTime.SimTimeUnit;
import simulator.elevatormodules.passenger.PassengerControl;
import simulator.framework.*;

/**
 * A static class that instantiates all controllers, sensors, and actuators
 * that are necessary for the elevator simulation.
 *
 * @param numFloors Number of floors in the simulation.
 * @param motorVerbose  Tell the doors to operate in verbose mode or not.
 * @param networkVerbose Tell the doors to operate in verbose mode or not.
 * @param driveVerbose Tell the doors to operate in verbose mode or not.
 * @param weightVerbose
 * Tell the weight sensor to operate in verbose mode or not.
 */
public class Modules {

    /* Sensor and actuator periods:
     */
    public final static SimTime EMPTY_SENSOR_PERIOD  = new SimTime(500, SimTimeUnit.MILLISECOND);
    public final static SimTime POSITION_SENSOR_PERIOD  = new SimTime(50, SimTimeUnit.MILLISECOND);
    public final static SimTime VEND_ACTUATOR_PERIOD  = new SimTime(10, SimTimeUnit.MILLISECOND);
    public final static SimTime MOTOR_ACTUATOR_PERIOD  = new SimTime(10, SimTimeUnit.MILLISECOND);
    public final static SimTime BUTTON_PRESS_TIME  = new SimTime(100, SimTimeUnit.MILLISECOND);
    public final static SimTime BUTTON_PERIOD  = new SimTime(10, SimTimeUnit.MILLISECOND);
    public final static SimTime RETURN_BUTTON_PERIOD  = new SimTime(100, SimTimeUnit.MILLISECOND);
    public final static SimTime CHANGE_MACHINE_PERIOD  = new SimTime(10, SimTimeUnit.MILLISECOND);
    


    /**
     * Distance in meters between floor landings.
     */
    public final static double FLOOR_DISTANCE = 5.0;

    /**
     * This class is a static factory, so it has no constructor visible
     * outside this class.
     */
    private Modules() {
    }

    public static PassengerControl makeAll(boolean verbose) {
        ChangeMachine cm = new ChangeMachine(CHANGE_MACHINE_PERIOD, verbose);
        VendActuator va = new VendActuator(VEND_ACTUATOR_PERIOD, verbose);
        CoinReturnButton crb = new CoinReturnButton(BUTTON_PRESS_TIME, RETURN_BUTTON_PERIOD, verbose);
        new VendMotor(MOTOR_ACTUATOR_PERIOD, verbose);

        SodaButton[] sb = new SodaButton[SodaMachine.NUM_CHUTES];
        ButtonLight[] bl = new ButtonLight[SodaMachine.NUM_CHUTES];
        for (int i=0; i < SodaMachine.NUM_CHUTES; i++) {
            int index = i+1;
            new EmptySensor(EMPTY_SENSOR_PERIOD, index);
            sb[i] = new SodaButton(index, BUTTON_PRESS_TIME, BUTTON_PERIOD, verbose);
            bl[i] = new ButtonLight(index, verbose);
            new VendPositionSensor(POSITION_SENSOR_PERIOD, index);
        }

        PassengerControl pc = new PassengerControl(sb, bl, crb, cm, va);

        return pc;
    }
}
