package simulator.elevatormodules;

import jSimPack.SimTime;
import simulator.elevatorcontrol.MessageDictionary;
import simulator.framework.*;
import simulator.payloads.*;

/**
 * Detects whether the VC is in range of the chute that corresponds to the index.
 * There is one sensor for each chute.
 */

public class VendPositionSensor extends Module
{

    private double leftPosition;
    private double rightPosition;
    
    private final static int sensorWidth = 80; //mm

    private VendCarriagePositionPayload carPosition;
    private VendPositionPayload localState;
    private CanMailbox localStateNwk;
    private VendPositionCanPayloadTranslator localStateTranslator;
    private boolean previousValue = false;

    /**
     * Creates a sensor that detects when the car is near the specified floor.
     * When the Car position is between <code>bottom</code> and
     * <code>top</code>, inclusive, the Car is considered to be "at" this
     * floor.
     *
     * @param period
     * time in microseconds between AtFloor messages
     */
    public VendPositionSensor (SimTime period, int index) {
        super(period, "VendPosition["+index+"]", false);

        double center = (index-1)*100; //compute the center position
        leftPosition = center - (sensorWidth/2);
        rightPosition = center + (sensorWidth/2);

        carPosition = new VendCarriagePositionPayload();
        localStateNwk = new CanMailbox(MessageDictionary.VEND_POSITION_BASE_CAN_ID + ReplicationComputer.computeReplicationId(index));
        localStateTranslator = new VendPositionCanPayloadTranslator(localStateNwk, index);

        localState = new VendPositionPayload(index);
        localState.value = false;

        physicalConnection.registerEventTriggered(carPosition);

        canNetworkConnection.sendTimeTriggered(localStateNwk, period);
        physicalConnection.sendTimeTriggered(localState, period);
        physicalConnection.sendOnce(localState);
    }
    
    @Override
    public void receive(VendCarriagePositionPayload msg)
    {
        //get position in mm
        double position = carPosition.position;

        localState.value = (leftPosition <= position) && (position <= rightPosition);

        if(previousValue != localState.value) {
            physicalConnection.sendOnce(localState);
        }
        //copy to network
        localStateTranslator.setValue(localState.value);
        previousValue = localState.value;
    }
}
