package simulator.framework;

import java.io.*;
import java.util.*;
import java.text.ParseException;

public class FileTokenizer
{

  boolean verbose;
  String filename;
  LineNumberReader theReader;
  Parser parser;
  String name;

  public FileTokenizer(String filename, boolean verbose, Parser parser) {

      this.filename = filename;
      this.verbose = verbose;
      this.parser = parser;
      name = "FileTokenizer("+filename+")";
  }

  public void parseFile()
  {
      try {
          theReader = new LineNumberReader(new FileReader(filename));

          log("opened "+filename+", reading...");

          for( String line = theReader.readLine(); null!=line; line = theReader.readLine() ){

              // delete comments, which is anything after and including the first
              // ';'
              line = line.replaceFirst("\\s*;.*", "").trim();

              if( line.length()==0 ){
                  log(lineMessage("skipping blank line"));
                  continue;
              }

              // strip whitespace from both ends, then split into space-separated
              // words
              String[] words = line.split("\\s+");
              log(lineMessage(String.format(
                    "%d words: %s", words.length, Arrays.toString(words))));

              try {
                  parser.parse(words);
              } catch(ParseException e) {
                  //System.err.println(lineMessage(e.toString()));
                  throw new RuntimeException(lineMessage(e.getMessage()));
              }
          }

          theReader.close();
      } catch (FileNotFoundException e) {
          throw new IllegalArgumentException(e);
      } catch (IOException e) {
          throw new RuntimeException(e);
      }
  }
  
  private void log(String s)
  {
      if(verbose) Harness.log(name,s);
  }

  public String lineMessage(String s) {

      return String.format("%s:%d: %s",
              filename, theReader.getLineNumber(), s);
  }

}
