package simulator.framework;

import jSimPack.SimTime;
import simulator.payloads.*;

/**
 * Connects elevator components to the physical framework. This class allows
 * elevator controllers to send and receive messages to and from framework
 * components, such as the buttons, doors, and drive motor. The component must
 * send messages of the same type it receives, although it may choose to only
 * perform only of those two functions. If the component tries to send or
 * receive two messages of different types, or it tries to send a message of a
 * different type than it registers to receive, a runtime exception will occur.
 */
public class PhysicalNetwork extends NetworkScheduler
{
    public class PhysicalConnection //implements NetworkConnection
    {

        private NetworkConnection conn;
        private boolean isRegistered;
        private boolean isSending;

        public PhysicalConnection(Networkable connectedTo)
        {
            conn = new Connection(connectedTo);
            isRegistered = false;
            isSending = false;
        }
        
        public PhysicalConnection()
        {
            this(null);
        }

        public void registerTimeTriggered(PhysicalPayload writeback)
        {
            if (isRegistered)
                throw new RuntimeException(
                "already registered for a physical framework message.");

            isRegistered = true;
            conn.registerTimeTriggered(writeback);
        }

        
        /*public void registerEventTriggered(Payload writeback)
        {
            if (isRegistered)
                throw new RuntimeException(
                "already registered for a physical framework message.");

            isRegistered = true;
            conn.registerEventTriggered(writeback);
        }*/

        public void sendTimeTriggered(PhysicalPayload msg, SimTime period)
        {
            if (isSending)
                throw new RuntimeException(
                "already sending a time-triggered message.");

            isSending = true;
            conn.sendTimeTriggered(msg, period);
        }

        /*@Override
        public void sendOnce(Payload msg)
        {
            if (isSending && msg.getType() != typeSent)
                throw new IllegalArgumentException(
                "cannot send more than one type of message");

            isSending = true;
            typeSent = msg.getType();
            conn.sendOnce(msg);
        }
        public void checkPayload(Payload payload) {
            if (!(payload instanceof PhysicalPayload)) {
                throw new RuntimeException("Payloads to the Physical network must be PhysicalPayloads");
            }
        }*/
        
        public void setEnabled(boolean enabled)
        {
            conn.setEnabled(enabled);
        }
    }
    
    public PhysicalNetwork()
    {
        super();
    }

    public PhysicalConnection getConnection(Networkable networkNode)
    {
        return new PhysicalConnection(networkNode);
    }
    
    public PhysicalConnection getConnection()
    {
        return new PhysicalConnection(null);
    }
    
    public Connection getFrameworkConnection()
    {
        return getFrameworkConnection(null);
    }

    public Connection getFrameworkConnection(Networkable networkNode)
    {
        return new Connection(networkNode);
    }
    
    @Override
    public String toString()
    {
        return "PhysNetwork";
    }
        
}