package simulator.payloads;

import simulator.framework.*;

/**
 * Network messages indicate whether a car call has been requested for
 * Hallway[f,b], and framework messages indicate whether the button is
 * currently held down.  Set to False at initialization in both cases.
 *
 * @author jdevale
 */
public class ButtonPayload extends PhysicalPayload {

    final public int index;
    public boolean pressed;

    /**
     * <code>floorNumber</code> is between 1 and {@link
     * simulator.framework.Elevator#numFloors}; <code>hallway</code> is either
     * <code>Hallway.FRONT</code> or <code>Hallway.BACK</code>
     */
    public ButtonPayload(int index) {
        super(PhysicalPayload.ButtonEvent, ReplicationComputer.computeReplicationId(index));
        this.index = index;
        this.pressed = false;
        setName("ButtonPayload(" + index + ")");
    }

    public ButtonPayload set(boolean buttonValue) {
        this.pressed = buttonValue;
        return this;
    }

    @Override
    public void copyFrom(Payload p) {
        super.copyFrom(p);
        ButtonPayload c = (ButtonPayload) p;
        if (this.index != c.index) {
            throw new RuntimeException("Wrong replication instance of " + this);
        }
        pressed = c.pressed;
    }

    /**
     * Returns a String that represents the value of this object.
     * @return a string representation of the receiver
     */
    @Override
    public String toString() {
        return super.toString() + "(" + pressed + ")";
    }

    @Override
    public void deliverTo(Networkable networkable) {
        networkable.receive(this);
    }

    @Override
    public Payload clone() {
        ButtonPayload c = new ButtonPayload(this.index);
        c.copyFrom(this);
        return c;
    }
}
