package simulator.payloads;

import simulator.framework.*;

/**
 * Framework state that for the coin reciever.  Pulsed true when the coin is received.
 */
public class CoinInPayload extends PhysicalPayload {

    public boolean value;

    /**
     * <code>floorNumber</code> is between 1 and {@link
     * simulator.framework.Elevator#numFloors}; <code>hallway</code> is either
     * <code>Hallway.FRONT</code> or <code>Hallway.BACK</code>
     */
    public CoinInPayload() {
        super(PhysicalPayload.CoinInEvent);
        this.value = false;
        setName("CoinInPayload");

    }

    public CoinInPayload set(boolean buttonValue) {
        this.value = buttonValue;
        return this;
    }

    @Override
    public void copyFrom(Payload p) {
        super.copyFrom(p);
        CoinInPayload c = (CoinInPayload) p;
        value = c.value;
    }

    /**
     * Returns a String that represents the value of this object.
     * @return a string representation of the receiver
     */
    @Override
    public String toString() {
        return super.toString() + "(" + value + ")";
    }

    @Override
    public void deliverTo(Networkable networkable) {
        networkable.receive(this);
    }

    @Override
    public Payload clone() {
        CoinInPayload c = new CoinInPayload();
        c.copyFrom(this);
        return c;
    }
}
