package simulator.payloads;

import simulator.framework.*;

/**
 * Framework message commands the vend motor to move the VC left and right on
 * its track.  Set to (Stop) at initialization.
 *
 * @author jdevale
 */
public class CoinTrayPayload extends PhysicalPayload {

    public int coinCount;

    public CoinTrayPayload() {
        super(PhysicalPayload.CoinTrayEvent);
        coinCount = 0;
        setName("CoinTrayPayload");
    }

    public void set(int sodaValue) {
        this.coinCount = sodaValue;
    }

    public void increment() {
        coinCount++;
    }

    public void decrement() {
        coinCount--;
    }

    public int getCoinCount() {
        return coinCount;
    }

    @Override
    public void copyFrom(Payload src) {
        super.copyFrom(src);
        coinCount = ((CoinTrayPayload) src).coinCount;
    }

    @Override
    public String toString() {
        return super.toString() + "(" + coinCount + ")";
    }

    @Override
    public void deliverTo(Networkable networkable) {
        networkable.receive(this);
    }

    @Override
    public Payload clone() {
        CoinTrayPayload c = new CoinTrayPayload();
        c.copyFrom(this);
        return c;
    }
}
