package simulator.payloads;

import simulator.framework.*;

/**
 * Network messages indicate whether a car call has been requested for
 * Hallway[f,b], and framework messages indicate whether the button is
 * currently held down.  Set to False at initialization in both cases.
 *
 * @author jdevale
 */
public class SodaChutePayload extends PhysicalPayload {

    final public int index;
    public int sodaCount;

    /**
     * <code>floorNumber</code> is between 1 and {@link
     * simulator.framework.Elevator#numFloors}; <code>hallway</code> is either
     * <code>Hallway.FRONT</code> or <code>Hallway.BACK</code>
     */
    public SodaChutePayload(int index) {
        super(PhysicalPayload.SodaChuteEvent, ReplicationComputer.computeReplicationId(index));
        this.index = index;
        setName("ButtonPayload(" + index + ")");

    }

    public SodaChutePayload set(int sodaCount) {
        this.sodaCount = sodaCount;
        return this;
    }

    public int getSodaCount() {
        return sodaCount;
    }

    @Override
    public void copyFrom(Payload p) {
        super.copyFrom(p);
        SodaChutePayload c = (SodaChutePayload) p;
        if (this.index != c.index) {
            throw new RuntimeException("Wrong replication instance of " + this);
        }
        sodaCount = c.getSodaCount();
    }

    /**
     * Returns a String that represents the value of this object.
     * @return a string representation of the receiver
     */
    @Override
    public String toString() {
        return super.toString() + "(" + sodaCount + ")";
    }

    @Override
    public void deliverTo(Networkable networkable) {
        networkable.receive(this);
    }

    @Override
    public Payload clone() {
        SodaChutePayload c = new SodaChutePayload(this.index);
        c.copyFrom(this);
        return c;
    }
}
