package simulator.payloads;

import simulator.framework.*;

/**
 * Network messages indicate whether a car call has been requested for
 * Hallway[f,b], and framework messages indicate whether the button is
 * currently held down.  Set to False at initialization in both cases.
 *
 * @author jdevale
 */
public class VendActuatorPayload extends PhysicalPayload {

    public boolean active;

    /**
     * <code>floorNumber</code> is between 1 and {@link
     * simulator.framework.Elevator#numFloors}; <code>hallway</code> is either
     * <code>Hallway.FRONT</code> or <code>Hallway.BACK</code>
     */
    public VendActuatorPayload() {
        super(PhysicalPayload.VendActuatorEvent);
        this.active = false;
        setName("VendAcutatorPayload");

    }

    public VendActuatorPayload set(boolean buttonValue) {
        this.active = buttonValue;
        return this;
    }

    @Override
    public void copyFrom(Payload p) {
        super.copyFrom(p);
        VendActuatorPayload c = (VendActuatorPayload) p;
        active = c.active;
    }

    /**
     * Returns a String that represents the value of this object.
     * @return a string representation of the receiver
     */
    @Override
    public String toString() {
        return super.toString() + "(" + active + ")";
    }

    @Override
    public void deliverTo(Networkable networkable) {
        networkable.receive(this);
    }

    @Override
    public Payload clone() {
        VendActuatorPayload c = new VendActuatorPayload();
        c.copyFrom(this);
        return c;
    }
}
