package simulator.payloads;

import simulator.framework.*;

/**
 * Framework message commands the vend motor to move the VC left and right on
 * its track.  Set to (Stop) at initialization.
 *
 * @author jdevale
 */
public class VendMotorPayload extends PhysicalPayload {

    public Direction direction;

    public VendMotorPayload() {
        super(PhysicalPayload.MotorEvent);
        direction = Direction.STOP;
        setName("MotorPayload");
    }

    public void set(Direction direction) {
        this.direction = direction;
    }

    public Direction direction() {
        return direction;
    }

    public Direction direction(Direction direction) {
        this.direction = direction;
        return direction;
    }

    @Override
    public void copyFrom(Payload src) {
        super.copyFrom(src);
        direction = ((VendMotorPayload) src).direction;
    }

    @Override
    public String toString() {
        return super.toString() + "(" + direction + ")";
    }

    @Override
    public void deliverTo(Networkable networkable) {
        networkable.receive(this);
    }

    @Override
    public Payload clone() {
        VendMotorPayload c = new VendMotorPayload();
        c.copyFrom(this);
        return c;
    }
}
